#include "main.h"
#include "maths.h"


/* draw an object */
void drawtriangle(float *v1, float *v2, float *v3)
{
	glBegin(GL_TRIANGLES);
	glNormal3fv(v1);
	glTexCoord3fv(v1);
	glVertex3fv(v1);
	glNormal3fv(v2);
	glTexCoord3fv(v2);
	glVertex3fv(v2);
	glNormal3fv(v3);
	glTexCoord3fv(v3);
	glVertex3fv(v3);
	glEnd();
}

/* calculate normals */
void normalize(float v[3])
{
	/* calculate length of vector */
	GLfloat lenght = sqrt(SQUARE(v[0]) + SQUARE(v[1]) + SQUARE(v[2]));
	if(lenght == 0) {
		printf("Vector has length zero, but that's... impossible!\n");
		exit(0);
	}
	v[0] /= lenght;
	v[1] /= lenght;
	v[2] /= lenght;
}

/* Subdivides a sphere (is it a Dodecahedron?) recrusivly. chech arg 4 */
void subdivide(float *v1, float *v2, float *v3, long depth)
{
	GLfloat v12[3], v23[3], v31[3];
	GLint i;
	if(depth == 0) {
		drawtriangle(v1, v2, v3);
		return;
	}
	for(i = 0; i < 3; i++) {
		v12[i] = (v1[i]+v2[i])/2.0;
		v23[i] = (v2[i]+v3[i])/2.0;
		v31[i] = (v3[i]+v1[i])/2.0;
	}
	normalize(v12);
	normalize(v23);
	normalize(v31);
	subdivide(v1, v12, v31, depth-1);
	subdivide(v2, v23, v12, depth-1);
	subdivide(v3, v31, v23, depth-1);
	subdivide(v12, v23, v31, depth-1);
}

/* Calculations for the Super Ellipse */
void evalsuperellipse(double t1, double t2, double p1, double p2, XYZ *p)
{
	double tmp;
	double ct1, ct2, st1, st2;
	ct1 = cos(t1);
	ct2 = cos(t2);
	st1 = sin(t1);
	st2 = sin(t2);
	tmp  = SIGN(ct1) * pow(fabs(ct1),p1);
	p->x = tmp * SIGN(ct2) * pow(fabs(ct2),p2);
	p->y = SIGN(st1) * pow(fabs(st1),p1);
	p->z = tmp * SIGN(st2) * pow(fabs(st2),p2);
}

/* Creates the Super Ellipse surface */
void createsuperellipse(double pow1, double pow2, int n)
{
	int i,j;
	double theta1, theta2, theta3, delta;
	XYZ p, p1, p2, en;
	delta = 0.01 * TWOPI / n;
	for(j = 0; j<n/2; j++) {
		theta1 = j * TWOPI / (double)n - PID2;
		theta2 = (j+1) * TWOPI / (double)n - PID2;
		glBegin(GL_QUAD_STRIP);
		for(i = 0; i<=n; i++) {
			if(i == 0 || i == n)
				theta3 = 0;
			else
				theta3 = i * TWOPI / n;
			evalsuperellipse(theta2,theta3,pow1,pow2,&p);
			evalsuperellipse(theta2+delta,theta3,pow1,pow2,&p1);
			evalsuperellipse(theta2,theta3+delta,pow1,pow2,&p2);
			glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
			glVertex3f(p.x,p.y,p.z);
			evalsuperellipse(theta1,theta3,pow1,pow2,&p);
			evalsuperellipse(theta1+delta,theta3,pow1,pow2,&p1);
			evalsuperellipse(theta1,theta3+delta,pow1,pow2,&p2);
			glTexCoord2f(i/(double)n,2*j/(double)n);
			glVertex3f(p.x,p.y,p.z);
		}
		glEnd();
	}
}

/* used in harmo() */
XYZ Eval(double theta, double phi)
{
   double r = 0;
	 double sinea = sin(rut/20000)*72;
	 double omg = sin(rut/400);
	 float beat = sin(rut/600)*4;
   XYZ p;
   r += pow(sin(2*phi+sinea),(float)4);
   r += pow(cos(2*phi+sinea),(float)4);
   r += pow(sin(2*theta+sinea),(float)4);
   r += pow(cos(2*theta+sinea),(float)4);
   p.x = r * sin(phi) * cos(theta);
   p.y = r * cos(phi);
   p.z = r * sin(phi) * sin(theta);
   return(p);
}

XYZ Eval2(double theta, double phi)
{
   double r = 0;
	 double sinea = sin(rut/20000)*72;
	 double omg = sin(rut/400);
	 float beat = sin(rut/600)*4;
   XYZ p;
   r += pow(sin(2*phi+sinea),(float)5);
   r += pow(cos(2*phi+sinea),(float)4);
   r += pow(sin(2*theta+sinea),(float)3);
   r += pow(cos(2*theta+sinea),(float)4);
   p.x = r * sin(phi) * cos(theta);
   p.y = r * cos(phi);
   p.z = r * sin(phi) * sin(theta);
   return(p);
}

/* Draws the Spherical Harmonics */
void harmo(double resolution)
{
	int i, j;
	double du = TWOPI / (double)resolution; /* Theta */
	double dv = PI / (double)resolution;    /* Phi   */
	glBegin(GL_QUADS);
	for (i=0;i<resolution;i++) {
		double u = i * du;
		for (j=0;j<resolution;j++) {
			double v = j * dv;
			q[0] = Eval(u,v);
			glTexCoord2f(i/resolution, 2*(j+1)/resolution);
			glVertex3f(q[0].x,q[0].y,q[0].z);
			q[1] = Eval(u+du,v);
			glTexCoord2f(i/resolution, 2*(j)/resolution);
			glVertex3f(q[1].x,q[1].y,q[1].z);
			q[2] = Eval(u+du,v+dv);
			glTexCoord2f(i/resolution, 2*(j+1)/resolution);
			glVertex3f(q[2].x,q[2].y,q[2].z);
			q[3] = Eval(u,v+dv);
			glTexCoord2f(i/resolution, 2*(j)/resolution);
			glVertex3f(q[3].x,q[3].y,q[3].z);
		}
	}
	glEnd();
}

/* Draws the Spherical Harmonics (blah) */
void harmo2(double resolution)
{
	int i, j;
	double du = TWOPI / (double)resolution; /* Theta */
	double dv = PI / (double)resolution;    /* Phi   */
	glBegin(GL_QUADS);
	for (i=0;i<resolution;i++) {
		double u = i * du;
		for (j=0;j<resolution;j++) {
			double v = j * dv;
			q[0] = Eval2(u,v);
			glTexCoord2f(i/resolution, 2*(j+1)/resolution);
			glVertex3f(q[0].x,q[0].y,q[0].z);
			q[1] = Eval2(u+du,v);
			glTexCoord2f(i/resolution, 2*(j)/resolution);
			glVertex3f(q[1].x,q[1].y,q[1].z);
			q[2] = Eval2(u+du,v+dv);
			glTexCoord2f(i/resolution, 2*(j+1)/resolution);
			glVertex3f(q[2].x,q[2].y,q[2].z);
			q[3] = Eval2(u,v+dv);
			glTexCoord2f(i/resolution, 2*(j)/resolution);
			glVertex3f(q[3].x,q[3].y,q[3].z);
		}
	}
	glEnd();
}
