#include "main.h"
#include "plane.h"
#include "circle.h"
#include "pyramid.h"
#include "cube.h"
#include "sphere.h"
#include "tg03.h"

#define STEPS 1000
#define SQUARE(x) ((x)*(x))

unsigned int ltime = 0;
int i;
float roty;
float tz = 5.0; /* translation along the z-axis */
unsigned int steps = 0.0;
float quadsize = 0.1;

particle particles[256];

 /* Move the particle closer to its destination */
void moveParticle(int n)
{
	/* Are we there yet? */
	if( sqrt( SQUARE(particles[n].dx - particles[n].x) + SQUARE(particles[n].dy - particles[n].y) + SQUARE(particles[n].dz - particles[n].z) ) <= 
			( sqrt(SQUARE(particles[n].vx) + SQUARE(particles[n].vy) + SQUARE(particles[n].vz))) ) {
		particles[n].x = particles[n].dx;
		particles[n].y = particles[n].dy; /* Close enough, just set position = destination */
		particles[n].z = particles[n].dz;
		
	}
	/* Not close enough, keep moving */
	else {
		particles[n].x += particles[n].vx;
		particles[n].y += particles[n].vy;
		particles[n].z += particles[n].vz;
	}
}

/* Draw the particle at position x,y,z */
void drawParticle(int n, float ab)
{
	glBegin(GL_QUADS);
	glTexCoord2f(0.0, 0.0); glVertex3f(particles[n].x-quadsize, particles[n].y-quadsize, particles[n].z);
	glTexCoord2f(1.0, 0.0); glVertex3f(particles[n].x+quadsize, particles[n].y-quadsize, particles[n].z);
	glTexCoord2f(1.0, 1.0); glVertex3f(particles[n].x+quadsize, particles[n].y+quadsize, particles[n].z);
	glTexCoord2f(0.0, 1.0); glVertex3f(particles[n].x-quadsize, particles[n].y+quadsize, particles[n].z);

	glTexCoord2f(0.0, 0.0); glVertex3f(particles[n].x, particles[n].y-quadsize, particles[n].z-quadsize);
	glTexCoord2f(1.0, 0.0); glVertex3f(particles[n].x, particles[n].y+quadsize, particles[n].z-quadsize);
	glTexCoord2f(1.0, 1.0); glVertex3f(particles[n].x, particles[n].y+quadsize, particles[n].z+quadsize);
	glTexCoord2f(0.0, 1.0); glVertex3f(particles[n].x, particles[n].y-quadsize, particles[n].z+quadsize);
	glEnd();
}

/* Calculate movement-vector */
void setCourse(int n)
{
	float vectorLength;

	particles[n].vx = particles[n].dx - particles[n].x;
	particles[n].vy = particles[n].dy - particles[n].y;
	particles[n].vz = particles[n].dz - particles[n].z;

	/* Calculate the vectors length */
	vectorLength = sqrt(SQUARE(particles[n].vx) + SQUARE(particles[n].vy) + SQUARE(particles[n].vz));

	particles[n].vx = particles[n].vx*(vectorLength / particles[n].steps);
	particles[n].vy = particles[n].vy*(vectorLength / particles[n].steps);	/* Make sure we get to the destination on time */
	particles[n].vz = particles[n].vz*(vectorLength / particles[n].steps);
}

void setPosition(float x, float y, float z, int n)
{
	particles[n].x = x;
	particles[n].y = y;
	particles[n].z = z;
}

void setDestination(float x, float y, float z, int n)
{
	particles[n].dx = x;
	particles[n].dy = y;
	particles[n].dz = z;
}

void setSteps(int steps, int n)
{
	particles[n].steps = steps;
}

void initPlaneParticles()
{
	for (i = 0; i < 256; i++) {
		setDestination(planeCoords[i][0], planeCoords[i][1], planeCoords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void initCircleParticles()
{
	for (i = 0; i < 256; i++) {
		setDestination(circleCoords[i][0], circleCoords[i][1], circleCoords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void initPyramidParticles()
{
	for (i = 0; i < 256; i++) {
		setDestination(pyramidCoords[i][0], pyramidCoords[i][1], pyramidCoords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void initCubeParticles()
{
	for (i = 0; i < 256; i++) {
		setDestination(cubeCoords[i][0], cubeCoords[i][1], cubeCoords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void initSphereParticles()
{
	for (i = 0; i < 256; i++) {
		setDestination(sphereCoords[i][0], sphereCoords[i][1], sphereCoords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void initTg03Particles()
{
	for (i = 0; i < 256; i++) {
		setDestination(tg03Coords[i][0], tg03Coords[i][1], tg03Coords[i][2], i);
		setSteps(STEPS, i);
		setCourse(i);
	}
}

void scene4()
{
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glScalef(1.0, 1.0, 1.0);
	BINDTEX(10);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glTranslatef(0.0, 0.0,-tz);
	glRotatef(roty, 0.0, 1.0, 0.0);
	if (steps == 0) {
		for (i = 0; i < 256; i++)
			setPosition((rand()%6)-3, (rand()%6)-3, (rand()%6)-3, i);
		initPlaneParticles();
	}

	if(glutGet(GLUT_ELAPSED_TIME)-ltime > 10) {
		for(i=0;i<256;i++)
			moveParticle(i);
		steps++;
		ltime = glutGet(GLUT_ELAPSED_TIME);
	}

	for (i = 0; i < 256; i++) {
		drawParticle(i, 197+(-roty/256));
	}
	if (steps == STEPS)
		initCircleParticles();
	else if (steps == STEPS*2)
		initPyramidParticles();
	else if (steps == STEPS*3)
		initCubeParticles();
	else if (steps == STEPS*4)
		initSphereParticles();
	else if (steps == STEPS*5)
		initTg03Particles();
	else if (steps == STEPS*6)
		setPosition((rand()%6)-3, (rand()%6)-3, (rand()%6)-3, i);

	if(steps > STEPS*4 && steps < STEPS*5) {
		quadsize += 0.00019;
		tz += 0.010;
	}
	roty += 0.1;
}

